#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import yacm

import pytest

import yacm

BASE = os.path.dirname(__file__)


def test_merge():
    # check to merge two config files
    foo = {'c': {'foo': 1}}
    cfg = {}
    yacm.merge(cfg, foo)
    assert cfg == foo

    yacm.merge(cfg, {})
    assert cfg == foo

    # configs must be dict of dicts
    with pytest.raises(AttributeError):
        yacm.merge({}, 1)

    with pytest.raises(AttributeError):
        yacm.merge({}, {'a': 1})


def test_simple():
    cfg = yacm.read_file(BASE + '/configs/simple.yml')
    assert isinstance(cfg, dict)
    assert cfg['plugins']['db'] is True

    cfg = yacm.read_file([
        BASE + '/configs/simple.yml',
        BASE + '/configs/no_rwdb.yml'
    ])
    assert cfg['plugins']['db'] is False
    assert cfg['plugins']['someother_db'] is True

    cfg = yacm.read_file([
        BASE + '/configs/no_rwdb.yml',
        BASE + '/configs/simple.yml',
    ])
    assert cfg['plugins']['db'] is True
    assert cfg['plugins']['someother_db'] is True


def test_config_paths():
    """when inside a virtualenv we are looking for more configs"""
    env = os.environ.get('VIRTUAL_ENV')
    if env is not None:
        del os.environ['VIRTUAL_ENV']
    configs = yacm.get_config_paths('rw')
    os.environ['VIRTUAL_ENV'] = '/tmp'
    configs_ve = yacm.get_config_paths('rw')
    if env is None:
        del os.environ['VIRTUAL_ENV']
    else:
        os.environ['VIRTUAL_ENV'] = env

    assert len(configs) < len(configs_ve)



# Copyright 2014-2018 Florian Ludwig
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Load yaml based configuration"""

__version__ = '0.1.0'

import os
import pkg_resources
import logging

import yaml


LOG = logging.getLogger(__name__)


def read_file(paths) -> dict:
    """read config from path or list of paths

    :param str|list[str] paths: path or list of paths
    :return dict: loaded and merged config
    """

    if isinstance(paths, str):
        paths = [paths]

    re = {}
    for path in paths:
        cfg = yaml.load(open(path))
        merge(re, cfg)

    return re


def merge(base_config:dict, additional_config:dict):
    """merge `additional_config` into `base_config`"""
    for category, category_data in additional_config.items():
        if isinstance(category_data, dict):
            base_config.setdefault(category, {})
            for key, value in category_data.items():
                base_config[category][key] = value
        else:
            raise AttributeError(
                'Config files must be in format {category: {key: value, ...}, ...}')


def get_config_paths(module_name) -> []:
    """determine configs to load"""
    cfg_name = module_name + '.yml'
    paths = []
    paths += [
        pkg_resources.resource_filename(module_name, 'config_default.yml'),
        '/etc/' + cfg_name,
        os.path.expanduser('~/.') + cfg_name
    ]
    if 'VIRTUAL_ENV' in os.environ:
        paths.append(os.environ['VIRTUAL_ENV'] + '/etc/' + cfg_name)
    return paths


def read_configs(module_name, extra_configs=None) -> dict:
    """
    """
    cfg = {}
    paths = get_config_paths(module_name)
    if extra_configs:
        if isinstance(extra_configs, list):
            paths.extend(extra_configs)
        else:
            paths.append(extra_configs)

    for path in paths:
        if os.path.exists(path):
            LOG.info('reading config: ' + path)
            merge(cfg, read_file(path))
    return cfg
